using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Object = UnityEngine.Object;

namespace CheesyFX
{
    public class FlexRegionSupportTracker
    {
        private Rigidbody rb;
        private FlexRegion flexRegion;
        private List<FlexCollisionTrigger> triggers = new List<FlexCollisionTrigger>();
        public HashSet<Rigidbody> stuffSupporting = new HashSet<Rigidbody>();
        private bool _supported;
        public bool colliding;

        public Func<float> GetUpAngle;
        public bool supported
        {
            get { return _supported; }
            set
            {
                if (_supported == value) return;
                _supported = flexRegion.supported = value;
            }
        }
        private float onStayTimeout;

        public FlexRegionSupportTracker(FlexRegion flexRegion, Rigidbody rigidbody, List<FlexCollisionTrigger> triggers)
        {
            rb = rigidbody;
            this.flexRegion = flexRegion;
            this.triggers = triggers;
            foreach (var trigger in triggers)
            {
                trigger.tracker = this;
            }

            if (flexRegion.id == 0) GetUpAngle = () => rb.transform.forward.y;
            else GetUpAngle = () => -rb.transform.up.y;
        }
        
        public void RegisterCollision(Collision collision)
        {
            if(!colliding) colliding = true;
            Vector3 contactVector = (collision.contacts[0].point - rb.position).normalized;
            float rbDown = GetUpAngle();
            bool supporting;
            supporting = contactVector.y < rbDown;
            if(supporting)
            {
                stuffSupporting.Add(collision.rigidbody);
                if(!supported)
                {
                    supported = true;
                    OnStayRoutine().Start();
                }
            }
        }
        
        public void DeregisterCollision(Collision collision){
            stuffSupporting.Remove(collision.rigidbody);
            if(stuffSupporting.Count == 0)
            {
                colliding = false;
                onStayTimeout = .05f;
            }
        }
        
        public IEnumerator OnStayRoutine(){
            while(stuffSupporting.Count > 0 || onStayTimeout > 0f)
            {
                if(stuffSupporting.Count == 0) onStayTimeout -= Time.deltaTime;
                yield return new WaitForSeconds(Time.fixedDeltaTime);
            }
            supported = false;
            // supported.Print();
        }

        public void Destroy()
        {
            for (int i = 0; i < triggers.Count; i++)
            {
                Object.Destroy(triggers[i]);
            }
        }
    }
}